@@ -1,16 +1,17 @@
-{-# LANGUAGE BangPatterns #-}
-{-# LANGUAGE DeriveFoldable #-}
-{-# LANGUAGE DeriveFunctor #-}
-{-# LANGUAGE DeriveTraversable #-}
-{-# LANGUAGE FlexibleContexts #-}
-{-# LANGUAGE FlexibleInstances #-}
-{-# LANGUAGE FunctionalDependencies #-}
-{-# LANGUAGE MultiParamTypeClasses #-}
-{-# LANGUAGE Rank2Types #-}
-{-# LANGUAGE TemplateHaskell #-}
+{-# language BangPatterns           #-}
+{-# language CPP                    #-}
+{-# language DeriveFoldable         #-}
+{-# language DeriveFunctor          #-}
+{-# language DeriveTraversable      #-}
+{-# language FlexibleContexts       #-}
+{-# language FlexibleInstances      #-}
+{-# language FunctionalDependencies #-}
+{-# language MultiParamTypeClasses  #-}
+{-# language Rank2Types             #-}
+{-# language TemplateHaskell        #-}
 -----------------------------------------------------------------------------
 -- |
--- Copyright   :  (c) Edward Kmett 2011-2015
+-- Copyright   :  (c) Edward Kmett 2011-2019
 -- License     :  BSD3
 --
 -- Maintainer  :  ekmett@gmail.com
@@ -29,6 +30,7 @@ module Text.Trifecta.Parser
   , stepResult
   , stepIt
   -- * Parsing
+  , runParser
   , parseFromFile
   , parseFromFileEx
   , parseString
@@ -42,24 +44,26 @@ import Control.Monad.IO.Class
 import qualified Control.Monad.Fail as Fail
 import Data.ByteString as Strict hiding (empty, snoc)
 import Data.ByteString.UTF8 as UTF8
-import Data.Maybe (isJust)
+import Data.Maybe (fromMaybe, isJust)
+#if !(MIN_VERSION_base(4,11,0))
 import Data.Semigroup
+#endif
 import Data.Semigroup.Reducer
 -- import Data.Sequence as Seq hiding (empty)
 import Data.Set as Set hiding (empty, toList)
+import Data.Text.Prettyprint.Doc as Pretty hiding (line)
 import System.IO
 import Text.Parser.Combinators
 import Text.Parser.Char
 import Text.Parser.LookAhead
 import Text.Parser.Token
-import Text.PrettyPrint.ANSI.Leijen as Pretty hiding (line, (<>), (<$>), empty)
 import Text.Trifecta.Combinators
-import Text.Trifecta.Instances ()
+import Text.Trifecta.Delta       as Delta
 import Text.Trifecta.Rendering
 import Text.Trifecta.Result
 import Text.Trifecta.Rope
-import Text.Trifecta.Delta as Delta
 import Text.Trifecta.Util.It
+import Text.Trifecta.Util.Pretty
 
 -- | The type of a trifecta parser
 --
@@ -94,52 +98,53 @@ import Text.Trifecta.Util.It
 -- if *all* parsers in the choice do.  If that is not the desired behavior,
 -- see `try`, which turns a committed parser failure into an epsilon failure
 -- (at the cost of error information).
---
 newtype Parser a = Parser
   { unparser :: forall r.
-    (a -> Err -> It Rope r) ->
-    (Err -> It Rope r) ->
-    (a -> Set String -> Delta -> ByteString -> It Rope r) -> -- committed success
-    (ErrInfo -> It Rope r) ->                                -- committed err
-    Delta -> ByteString -> It Rope r
+       (a -> Err -> It Rope r)
+    -> (Err -> It Rope r)
+    -> (a -> Set String -> Delta -> ByteString -> It Rope r)  -- committed success
+    -> (ErrInfo -> It Rope r)                                 -- committed err
+    -> Delta
+    -> ByteString
+    -> It Rope r
   }
 
 instance Functor Parser where
   fmap f (Parser m) = Parser $ \ eo ee co -> m (eo . f) ee (co . f)
-  {-# INLINE fmap #-}
+  {-# inlinable fmap #-}
   a <$ Parser m = Parser $ \ eo ee co -> m (\_ -> eo a) ee (\_ -> co a)
-  {-# INLINE (<$) #-}
+  {-# inlinable (<$) #-}
 
 instance Applicative Parser where
   pure a = Parser $ \ eo _ _ _ _ _ -> eo a mempty
-  {-# INLINE pure #-}
+  {-# inlinable pure #-}
   (<*>) = ap
-  {-# INLINE (<*>) #-}
+  {-# inlinable (<*>) #-}
 
 instance Alternative Parser where
   empty = Parser $ \_ ee _ _ _ _ -> ee mempty
-  {-# INLINE empty #-}
+  {-# inlinable empty #-}
   Parser m <|> Parser n = Parser $ \ eo ee co ce d bs ->
     m eo (\e -> n (\a e' -> eo a (e <> e')) (\e' -> ee (e <> e')) co ce d bs) co ce d bs
-  {-# INLINE (<|>) #-}
+  {-# inlinable (<|>) #-}
   many p = Prelude.reverse <$> manyAccum (:) p
-  {-# INLINE many #-}
+  {-# inlinable many #-}
   some p = (:) <$> p <*> Alternative.many p
 
 instance Semigroup a => Semigroup (Parser a) where
   (<>) = liftA2 (<>)
-  {-# INLINE (<>) #-}
+  {-# inlinable (<>) #-}
 
 instance (Semigroup a, Monoid a) => Monoid (Parser a) where
   mappend = (<>)
-  {-# INLINE mappend #-}
+  {-# inlinable mappend #-}
 
   mempty = pure mempty
-  {-# INLINE mempty #-}
+  {-# inlinable mempty #-}
 
 instance Monad Parser where
   return = pure
-  {-# INLINE return #-}
+  {-# inlinable return #-}
   Parser m >>= k = Parser $ \ eo ee co ce d bs ->
     m -- epsilon result: feed result to monadic continutaion; committed
       -- continuations as they were given to us; epsilon callbacks merge
@@ -169,21 +174,23 @@ instance Monad Parser where
          d' bs')
       -- committed error, delta, and bytestring: as given
       ce d bs
-  {-# INLINE (>>=) #-}
+  {-# inlinable (>>=) #-}
   (>>) = (*>)
-  {-# INLINE (>>) #-}
+  {-# inlinable (>>) #-}
+#if !(MIN_VERSION_base(4,13,0))
   fail = Fail.fail
-  {-# INLINE fail #-}
+  {-# inlinable fail #-}
+#endif
 
 instance Fail.MonadFail Parser where
   fail s = Parser $ \ _ ee _ _ _ _ -> ee (failed s)
-  {-# INLINE fail #-}
+  {-# inlinable fail #-}
 
 instance MonadPlus Parser where
   mzero = empty
-  {-# INLINE mzero #-}
+  {-# inlinable mzero #-}
   mplus = (<|>)
-  {-# INLINE mplus #-}
+  {-# inlinable mplus #-}
 
 manyAccum :: (a -> [a] -> [a]) -> Parser a -> Parser [a]
 manyAccum f (Parser p) = Parser $ \eo _ co ce d bs ->
@@ -196,31 +203,31 @@ liftIt :: It Rope a -> Parser a
 liftIt m = Parser $ \ eo _ _ _ _ _ -> do
   a <- m
   eo a mempty
-{-# INLINE liftIt #-}
+{-# inlinable liftIt #-}
 
 instance Parsing Parser where
   try (Parser m) = Parser $ \ eo ee co _ -> m eo ee co (\_ -> ee mempty)
-  {-# INLINE try #-}
+  {-# inlinable try #-}
   Parser m <?> nm = Parser $ \ eo ee -> m
      (\a e -> eo a (if isJust (_reason e) then e { _expected = Set.singleton nm } else e))
      (\e -> ee e { _expected = Set.singleton nm })
-  {-# INLINE (<?>) #-}
+  {-# inlinable (<?>) #-}
   skipMany p = () <$ manyAccum (\_ _ -> []) p
-  {-# INLINE skipMany #-}
+  {-# inlinable skipMany #-}
   unexpected s = Parser $ \ _ ee _ _ _ _ -> ee $ failed $ "unexpected " ++ s
-  {-# INLINE unexpected #-}
+  {-# inlinable unexpected #-}
   eof = notFollowedBy anyChar <?> "end of input"
-  {-# INLINE eof #-}
+  {-# inlinable eof #-}
   notFollowedBy p = try (optional p >>= maybe (pure ()) (unexpected . show))
-  {-# INLINE notFollowedBy #-}
+  {-# inlinable notFollowedBy #-}
 
 instance Errable Parser where
   raiseErr e = Parser $ \ _ ee _ _ _ _ -> ee e
-  {-# INLINE raiseErr #-}
+  {-# inlinable raiseErr #-}
 
 instance LookAheadParsing Parser where
   lookAhead (Parser m) = Parser $ \eo ee _ -> m eo ee (\a _ _ _ -> eo a mempty)
-  {-# INLINE lookAhead #-}
+  {-# inlinable lookAhead #-}
 
 instance CharParsing Parser where
   satisfy f = Parser $ \ _ ee co _ d bs ->
@@ -233,27 +240,27 @@ instance CharParsing Parser where
                                               (co c mempty)
                                               ddc
         | otherwise       -> co c mempty (d <> delta c) bs
-  {-# INLINE satisfy #-}
+  {-# inlinable satisfy #-}
 
 instance TokenParsing Parser
 
 instance DeltaParsing Parser where
   line = Parser $ \eo _ _ _ _ bs -> eo bs mempty
-  {-# INLINE line #-}
+  {-# inlinable line #-}
   position = Parser $ \eo _ _ _ d _ -> eo d mempty
-  {-# INLINE position #-}
+  {-# inlinable position #-}
   rend = Parser $ \eo _ _ _ d bs -> eo (rendered d bs) mempty
-  {-# INLINE rend #-}
+  {-# inlinable rend #-}
   slicedWith f p = do
     m <- position
     a <- p
     r <- position
     f a <$> liftIt (sliceIt m r)
-  {-# INLINE slicedWith #-}
+  {-# inlinable slicedWith #-}
 
 instance MarkParsing Delta Parser where
   mark = position
-  {-# INLINE mark #-}
+  {-# inlinable mark #-}
   release d' = Parser $ \_ ee co _ d bs -> do
     mbs <- rewindIt d'
     case mbs of
@@ -264,10 +271,25 @@ instance MarkParsing Delta Parser where
             else co () mempty d' mempty
         | otherwise -> ee mempty
 
+-- | A 'Step' allows for incremental parsing, since the parser
+--
+--   - can be done with a final result
+--   - have errored
+--   - can have yielded a partial result with possibly more to come
 data Step a
   = StepDone !Rope a
+    -- ^ Parsing is done and has converted the 'Rope' to a final result
+
   | StepFail !Rope ErrInfo
+    -- ^ Parsing the 'Rope' has failed with an error
+
   | StepCont !Rope (Result a) (Rope -> Step a)
+    -- ^ The 'Rope' has been partially consumed and already yielded a 'Result',
+    -- and if more input is provided, more results can be produced.
+    --
+    -- One common scenario for this is to parse log files: after parsing a
+    -- single line, that data can already be worked with, but there may be more
+    -- lines to come.
 
 instance Show a => Show (Step a) where
   showsPrec d (StepDone r a) = showParen (d > 10) $
@@ -282,28 +304,32 @@ instance Functor Step where
   fmap _ (StepFail r xs)   = StepFail r xs
   fmap f (StepCont r z k)  = StepCont r (fmap f z) (fmap f . k)
 
+-- | Feed some additional input to a 'Step' to continue parsing a bit further.
 feed :: Reducer t Rope => t -> Step r -> Step r
 feed t (StepDone r a)    = StepDone (snoc r t) a
 feed t (StepFail r xs)   = StepFail (snoc r t) xs
 feed t (StepCont r _ k)  = k (snoc r t)
-{-# INLINE feed #-}
+{-# inlinable feed #-}
 
+-- | Assume all possible input has been given to the parser, execute it to yield
+-- a final result.
 starve :: Step a -> Result a
 starve (StepDone _ a)    = Success a
 starve (StepFail _ xs)   = Failure xs
 starve (StepCont _ z _)  = z
-{-# INLINE starve #-}
+{-# inlinable starve #-}
 
 stepResult :: Rope -> Result a -> Step a
 stepResult r (Success a)  = StepDone r a
 stepResult r (Failure xs) = StepFail r xs
-{-# INLINE stepResult #-}
+{-# inlinable stepResult #-}
 
 stepIt :: It Rope a -> Step a
 stepIt = go mempty where
-  go r (Pure a) = StepDone r a
-  go r (It a k) = StepCont r (pure a) $ \s -> go s (k s)
-{-# INLINE stepIt #-}
+  go r m = case simplifyIt m r of
+    Pure a -> StepDone r a
+    It a k -> StepCont r (pure a) $ \r' -> go r' (k r')
+{-# inlinable stepIt #-}
 
 data Stepping a
   = EO a Err
@@ -311,30 +337,51 @@ data Stepping a
   | CO a (Set String) Delta ByteString
   | CE ErrInfo
 
-stepParser :: Parser a -> Delta -> ByteString -> Step a
-stepParser (Parser p) d0 bs0 = go mempty $ p eo ee co ce d0 bs0 where
-  eo a e       = Pure (EO a e)
-  ee e         = Pure (EE e)
-  co a es d bs = Pure (CO a es d bs)
-  ce errInf    = Pure (CE errInf)
-  go r (Pure (EO a _))     = StepDone r a
-  go r (Pure (EE e))       = StepFail r $
-                              let errDoc = explain (renderingCaret d0 bs0) e
-                              in  ErrInfo errDoc (_finalDeltas e)
-  go r (Pure (CO a _ _ _)) = StepDone r a
-  go r (Pure (CE d))       = StepFail r d
-  go r (It ma k)           = StepCont r (case ma of
-                                EO a _     -> Success a
-                                EE e       -> Failure $
-                                  ErrInfo (explain (renderingCaret d0 bs0) e) (d0 : _finalDeltas e)
-                                CO a _ _ _ -> Success a
-                                CE d       -> Failure d
-                              ) (go <*> k)
-{-# INLINE stepParser #-}
+-- | Incremental parsing. A 'Step' can be supplied with new input using 'feed',
+-- the final 'Result' is obtained using 'starve'.
+stepParser
+    :: Parser a
+    -> Delta -- ^ Starting cursor position. Usually 'mempty' for the beginning of the file.
+    -> Step a
+stepParser (Parser p) d0 = joinStep $ stepIt $ do
+  bs0 <- fromMaybe mempty <$> rewindIt d0
+  go bs0 <$> p eo ee co ce d0 bs0
+ where
+  eo a e        = Pure (EO a e)
+  ee e          = Pure (EE e)
+  co a es d' bs = Pure (CO a es d' bs)
+  ce errInf     = Pure (CE errInf)
 
--- | @parseFromFile p filePath@ runs a parser @p@ on the
--- input read from @filePath@ using 'ByteString.readFile'. All diagnostic messages
--- emitted over the course of the parse attempt are shown to the user on the console.
+  go :: ByteString -> Stepping a -> Result a
+  go _   (EO a _)     = Success a
+  go bs0 (EE e)       = Failure $
+                          let errDoc = explain (renderingCaret d0 bs0) e
+                          in  ErrInfo errDoc (d0 : _finalDeltas e)
+  go _   (CO a _ _ _) = Success a
+  go _   (CE e)       = Failure e
+
+  joinStep :: Step (Result a) -> Step a
+  joinStep (StepDone r (Success a)) = StepDone r a
+  joinStep (StepDone r (Failure e)) = StepFail r e
+  joinStep (StepFail r e)           = StepFail r e
+  joinStep (StepCont r a k)         = StepCont r (join a) (joinStep <$> k)
+  {-# inlinable joinStep #-}
+
+-- | Run a 'Parser' on input that can be reduced to a 'Rope', e.g. 'String', or
+-- 'ByteString'. See also the monomorphic versions 'parseString' and
+-- 'parseByteString'.
+runParser
+    :: Reducer t Rope
+    => Parser a
+    -> Delta -- ^ Starting cursor position. Usually 'mempty' for the beginning of the file.
+    -> t
+    -> Result a
+runParser p d bs = starve $ feed bs $ stepParser p d
+{-# inlinable runParser #-}
+
+-- | @('parseFromFile' p filePath)@ runs a parser @p@ on the input read from
+-- @filePath@ using 'ByteString.readFile'. All diagnostic messages emitted over
+-- the course of the parse attempt are shown to the user on the console.
 --
 -- > main = do
 -- >   result <- parseFromFile numbers "digits.txt"
@@ -347,34 +394,45 @@ parseFromFile p fn = do
   case result of
    Success a  -> return (Just a)
    Failure xs -> do
-     liftIO $ displayIO stdout $ renderPretty 0.8 80 $ (_errDoc xs) <> linebreak
+     liftIO $ renderIO stdout $ renderPretty 0.8 80 $ (_errDoc xs) <> line'
      return Nothing
 
--- | @parseFromFileEx p filePath@ runs a parser @p@ on the
--- input read from @filePath@ using 'ByteString.readFile'. Returns all diagnostic messages
--- emitted over the course of the parse and the answer if the parse was successful.
+-- | @('parseFromFileEx' p filePath)@ runs a parser @p@ on the input read from
+-- @filePath@ using 'ByteString.readFile'. Returns all diagnostic messages
+-- emitted over the course of the parse and the answer if the parse was
+-- successful.
 --
 -- > main = do
 -- >   result <- parseFromFileEx (many number) "digits.txt"
 -- >   case result of
 -- >     Failure xs -> displayLn xs
 -- >     Success a  -> print (sum a)
--- >
-
 parseFromFileEx :: MonadIO m => Parser a -> String -> m (Result a)
 parseFromFileEx p fn = do
   s <- liftIO $ Strict.readFile fn
   return $ parseByteString p (Directed (UTF8.fromString fn) 0 0 0 0) s
 
--- | @parseByteString p delta i@ runs a parser @p@ on @i@.
+-- | Fully parse a 'UTF8.ByteString' to a 'Result'.
+--
+-- @parseByteString p delta i@ runs a parser @p@ on @i@.
+parseByteString
+    :: Parser a
+    -> Delta -- ^ Starting cursor position. Usually 'mempty' for the beginning of the file.
+    -> UTF8.ByteString
+    -> Result a
+parseByteString = runParser
 
-parseByteString :: Parser a -> Delta -> UTF8.ByteString -> Result a
-parseByteString p d inp = starve $ feed inp $ stepParser (release d *> p) mempty mempty
-
-parseString :: Parser a -> Delta -> String -> Result a
-parseString p d inp = starve $ feed inp $ stepParser (release d *> p) mempty mempty
+-- | Fully parse a 'String' to a 'Result'.
+--
+-- @parseByteString p delta i@ runs a parser @p@ on @i@.
+parseString
+    :: Parser a
+    -> Delta -- ^ Starting cursor position. Usually 'mempty' for the beginning of the file.
+    -> String
+    -> Result a
+parseString = runParser
 
 parseTest :: (MonadIO m, Show a) => Parser a -> String -> m ()
 parseTest p s = case parseByteString p mempty (UTF8.fromString s) of
-  Failure xs -> liftIO $ displayIO stdout $ renderPretty 0.8 80 $ (_errDoc xs) <> linebreak -- TODO: retrieve columns
+  Failure xs -> liftIO $ renderIO stdout $ renderPretty 0.8 80 $ (_errDoc xs) <> line' -- TODO: retrieve columns
   Success a  -> liftIO (print a)
