@@ -1,7 +1,7 @@
-{-# LANGUAGE CPP #-}
-{-# LANGUAGE FlexibleInstances #-}
+{-# LANGUAGE CPP                   #-}
+{-# LANGUAGE FlexibleInstances     #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
-{-# LANGUAGE TypeFamilies #-}
+{-# LANGUAGE TypeFamilies          #-}
 
 #ifndef MIN_VERSION_lens
 #define MIN_VERSION_lens(x,y,z) 1
@@ -9,7 +9,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Text.Trifecta.Util.IntervalMap
--- Copyright   :  (c) Edward Kmett 2011-2015
+-- Copyright   :  (c) Edward Kmett 2011-2019
 --                (c) Ross Paterson 2008
 -- License     :  BSD-style
 -- Maintainer  :  ekmett@gmail.com
@@ -54,19 +54,21 @@ module Text.Trifecta.Util.IntervalMap
 
 #if __GLASGOW_HASKELL__ < 710
 import Control.Applicative hiding (empty)
-import Data.Foldable (Foldable(foldMap))
+import Data.Foldable       (Foldable (foldMap))
 #endif
-import Control.Lens
-  hiding ((<|),(|>)
 #if MIN_VERSION_lens(4,13,0) && __GLASGOW_HASKELL__ >= 710
-  ,(:<)
+import Control.Lens hiding ((:<), (<|), (|>))
+#else
+import Control.Lens hiding ((<|), (|>))
 #endif
-  )
-import qualified Data.FingerTree as FT
-import Data.FingerTree (FingerTree, Measured(..), ViewL(..), (<|), (><))
-import Data.Semigroup
-import Data.Semigroup.Reducer
-import Data.Semigroup.Union
+import           Data.FingerTree
+    (FingerTree, Measured (..), ViewL (..), (<|), (><))
+import qualified Data.FingerTree        as FT
+#if !(MIN_VERSION_base(4,11,0))
+import           Data.Semigroup
+#endif
+import           Data.Semigroup.Reducer
+import           Data.Semigroup.Union
 
 ----------------------------------
 -- 4.8 Application: interval trees
@@ -250,4 +252,3 @@ greater _ _ = False
 fromList :: Ord v => [(v, v, a)] -> IntervalMap v a
 fromList = foldr ins empty where
   ins (lo, hi, n) = insert lo hi n
-
